@@ -37,6 +37,7 @@ pub enum EcCommands {
     PwmGetDuty = 0x0026,
     SetTabletMode = 0x0031,
     AutoFanCtrl = 0x0052,
+    GpioSet = 0x0092,
     GpioGet = 0x0093,
     I2cPassthrough = 0x009e,
     ConsoleSnapshot = 0x0097,
@@ -44,6 +45,7 @@ pub enum EcCommands {
     ChargeState = 0x00A0,
     ChargeCurrentLimit = 0x00A1,
     HibernationDelay = 0x00A8,
+    S0ixCounter = 0x00AB,
     /// List the features supported by the firmware
     GetFeatures = 0x000D,
     /// Force reboot, causes host reboot as well
@@ -52,6 +54,7 @@ pub enum EcCommands {
     RebootEc = 0x00D2,
     /// Get information about PD controller power
     UsbPdPowerInfo = 0x0103,
+    GetUptimeInfo = 0x0121,
     AdcRead = 0x0123,
     ApReset = 0x0125,
     LocateChip = 0x0126,
@@ -72,6 +75,8 @@ pub enum EcCommands {
     ChassisOpenCheck = 0x3E0F,
     /// Get information about historical chassis open/close (intrusion) information
     ChassisIntrusion = 0x3E09,
+    /// Control and check retimer modes (firmware update and compliance)
+    RetimerControl = 0x3E0A,
 
     /// Not used by this library
     AcpiNotify = 0xE10,
@@ -105,6 +110,8 @@ pub enum EcCommands {
     GetGpuPcie = 0x3E1E,
     /// Set gpu bay serial and program structure
     ProgramGpuEeprom = 0x3E1F,
+    /// Read board ID of specific ADC channel
+    ReadBoardId = 0x3E26,
 }
 
 pub trait EcRequest<R> {
