@@ -452,6 +452,18 @@ impl EcRequest<()> for EcRequestAutoFanCtrlV1 {
     }
 }
 
+#[repr(C, packed)]
+pub struct EcRequestGpioSetV0 {
+    pub name: [u8; 32],
+    pub value: u8,
+}
+
+impl EcRequest<()> for EcRequestGpioSetV0 {
+    fn command_id() -> EcCommands {
+        EcCommands::GpioSet
+    }
+}
+
 #[repr(C, packed)]
 pub struct EcRequestGpioGetV0 {
     pub name: [u8; 32],
@@ -641,6 +653,25 @@ impl EcRequest<EcResponseHibernationDelay> for EcRequesetHibernationDelay {
     }
 }
 
+#[repr(C, packed)]
+pub struct EcRequestS0ixCounter {
+    /// If 0x01 then reset the counter, otherwise get it
+    pub flags: u32,
+}
+
+#[repr(C, packed)]
+pub struct EcResponseS0ixCounter {
+    pub s0ix_counter: u32,
+}
+
+pub const EC_S0IX_COUNTER_RESET: u32 = 0x01;
+
+impl EcRequest<EcResponseS0ixCounter> for EcRequestS0ixCounter {
+    fn command_id() -> EcCommands {
+        EcCommands::S0ixCounter
+    }
+}
+
 /// Supported features
 #[derive(Debug, FromPrimitive)]
 pub enum EcFeatureCode {
@@ -863,6 +894,132 @@ impl EcRequest<EcResponseUsbPdPowerInfo> for EcRequestUsbPdPowerInfo {
     }
 }
 
+#[repr(usize)]
+#[derive(Debug, FromPrimitive)]
+pub enum EcResetFlag {
+    /// Other known reason
+    Other,
+    /// Reset pin asserted
+    ResetPin,
+    /// Brownout
+    Brownout,
+    /// Power-on reset
+    PowerOn,
+    /// Watchdog timer reset
+    Watchdog,
+    /// Soft reset trigger by core
+    Soft,
+    /// Wake from hibernate
+    Hibernate,
+    /// RTC alarm wake
+    RtcAlarm,
+    /// Wake pin triggered wake
+    WakePin,
+    /// Low battery triggered wake
+    LowBattery,
+    /// Jumped directly to this image
+    Sysjump,
+    /// Hard reset from software
+    Hard,
+    /// Do not power on AP
+    APOff,
+    /// Some reset flags preserved from previous boot
+    Preserved,
+    /// USB resume triggered wake
+    UsbResume,
+    /// USB Type-C debug cable
+    Rdd,
+    /// Fixed Reset Functionality
+    Rbox,
+    /// Security threat
+    Security,
+    /// AP experienced a watchdog reset
+    ApWatchdog,
+    /// Do not select RW in EFS. This enables PD in RO for Chromebox
+    StayInRo,
+    /// Jumped to this image by EFS
+    Efs,
+    /// Leave alone AP
+    ApIdle,
+    /// EC had power, then was reset
+    InitialPwr,
+    Count,
+}
+
+#[repr(u16)]
+#[derive(Debug, FromPrimitive)]
+pub enum ResetCause {
+    ResetUnknown = 0x0000,
+    /// Custom reason defined by a board.c or baseboard.c file
+    ResetBoardCustom,
+    /// Believe that the AP has hung
+    ResetHangReboot,
+    /// Reset by EC console command
+    ResetConsoleCommand,
+    /// Reset by EC host command
+    ResetHostCommand,
+    /// Keyboard module reset key combination
+    ResetKeyboardSysReset,
+    /// Keyboard module warm reboot
+    ResetKeyboardWarmBoot,
+    /// Debug module warm reboot
+    ResetDebugWarmBoot,
+    /// I cannot self-terminate. You must lower me into the steel
+    ResetApReq,
+    /// Reset as side-effect of startup sequence
+    ResetInit,
+    /// EC detected an AP watchdog event
+    ResetApWatchdog,
+
+    ShutdownPowerFail = 0x8000,
+    /// Forcing a shutdown as part of EC initialization
+    ShutdownInit,
+    /// Custom reason on a per-board basis.
+    ShutdownBoardCustom,
+    /// This is a reason to inhibit startup, not cause shut down.
+    ShutdownBatteryInhibit,
+    /// A power_wait_signal is being asserted
+    ShutdownWait,
+    /// Critical battery level.
+    ShutdownBatteryCritical,
+    /// Because you told me to.
+    ShutdownConsoleCommand,
+    /// Forcing a shutdown to effect entry to G3.
+    ShutdownG3,
+    /// Force shutdown due to over-temperature.
+    ShutdownThermal,
+    /// Force a chipset shutdown from the power button through EC
+    ShutdownButton,
+}
+
+#[repr(C)]
+pub struct EcRequestGetUptimeInfo {}
+
+#[repr(C, packed)]
+#[derive(Clone, Copy, Debug)]
+pub struct ApResetLogEntry {
+    pub reset_cause: u16,
+    pub reserved: u16,
+    pub reset_time_ms: u32,
+}
+
+#[repr(C, packed)]
+#[derive(Debug)]
+pub struct EcResponseGetUptimeInfo {
+    pub time_since_ec_boot: u32,
+    /// How often the AP was reset by the EC since last EC boot
+    /// The first AP boot may count as more than one
+    pub ap_resets_since_ec_boot: u32,
+    pub ec_reset_flags: u32,
+    pub recent_ap_resets: [ApResetLogEntry; 4],
+}
+
+impl EcRequest<EcResponseGetUptimeInfo> for EcRequestGetUptimeInfo {
+    fn command_id() -> EcCommands {
+        EcCommands::GetUptimeInfo
+    }
+}
+
 #[repr(C, packed)]
 pub struct EcRequestAdcRead {
     /// ADC Channel, specific to each mainboard schematic
@@ -1022,6 +1179,39 @@ impl EcRequest<EcResponseChassisIntrusionControl> for EcRequestChassisIntrusionC
     }
 }
 
+#[repr(u8)]
+pub enum RetimerControlMode {
+    /// AMD and Intel
+    EntryFwUpdateMode = 0x01,
+    /// AMD and Intel
+    ExitFwUpdateMode = 0x02,
+    /// Intel only
+    EnableComplianceMode = 0x04,
+    /// Intel only
+    DisableComplianceMode = 0x08,
+    /// Check if in FwUpdateMode
+    CheckStatus = 0x80,
+}
+
+#[repr(C, packed)]
+pub struct EcRequestRetimerControl {
+    /// 0 (right) or 1 (left)
+    pub controller: u8,
+    /// See RetimerControlMode
+    pub mode: u8,
+}
+
+#[repr(C, packed)]
+pub struct EcResponseRetimerControlStatus {
+    pub status: u8,
+}
+
+impl EcRequest<EcResponseRetimerControlStatus> for EcRequestRetimerControl {
+    fn command_id() -> EcCommands {
+        EcCommands::RetimerControl
+    }
+}
+
 #[repr(C, packed)]
 pub struct EcRequestReadPdVersionV0 {}
 
@@ -1419,13 +1609,14 @@ pub enum GpuPcieConfig {
 }
 
 #[repr(u8)]
-#[derive(Debug, FromPrimitive)]
+#[derive(Debug, FromPrimitive, PartialEq)]
 pub enum GpuVendor {
     Initializing = 0x00,
     FanOnly = 0x01,
     GpuAmdR23M = 0x02,
     SsdHolder = 0x03,
     PcieAccessory = 0x4,
+    NvidiaGn22 = 0x5,
 }
 
 #[repr(C, packed)]
@@ -1467,3 +1658,39 @@ impl EcRequest<EcResponseSetGpuSerial> for EcRequestSetGpuSerial {
         EcCommands::ProgramGpuEeprom
     }
 }
+
+#[repr(u8)]
+#[derive(Debug, Clone, Copy)]
+pub enum BoardIdType {
+    /// Mainboard - any system
+    Mainboard = 0,
+    /// Power button board - Framework 12
+    PowerButtonBoard = 1,
+    /// Touchpad - Framework 12, 13, 16
+    Touchpad = 2,
+    /// Audio Board - Framework 12, 13
+    AudioBoard = 3,
+    /// dGPU board - Framework 16
+    DGpu0 = 4,
+    /// dGPU board - Framework 16
+    DGpu1 = 5,
+}
+
+#[repr(C, packed)]
+pub struct EcRequestReadBoardId {
+    /// See BoardIdType
+    pub board_id_type: u8,
+}
+
+#[repr(C, packed)]
+#[derive(Clone, Copy, PartialEq, Eq, Debug)]
+pub struct EcResponseReadBoardId {
+    /// Board ID: -1 invalid, 0–14 valid, 15 not present
+    pub board_id: i8,
+}
+
+impl EcRequest<EcResponseReadBoardId> for EcRequestReadBoardId {
+    fn command_id() -> EcCommands {
+        EcCommands::ReadBoardId
+    }
+}
