@@ -16,6 +16,8 @@ use crate::smbios;
 use crate::uefi::shell_get_execution_break_flag;
 use crate::util::{self, Platform};
 
+use no_std_compat::time::Duration;
+
 use log::Level;
 use num_derive::FromPrimitive;
 
@@ -199,11 +201,11 @@ pub enum Framework16Adc {
 
 const BOARD_VERSION_COUNT: usize = 16;
 const BOARD_VERSION: [i32; BOARD_VERSION_COUNT] = [
-    85, 233, 360, 492, 649, 844, 965, 1094, 1380, 1562, 1710, 2040, 2197, 2557, 2766, 2814,
+    203, 409, 615, 821, 1028, 1234, 1440, 1646, 1853, 2059, 2265, 2471, 2678, 2884, 3090, 3300,
 ];
 
 const BOARD_VERSION_NPC_DB: [i32; BOARD_VERSION_COUNT] = [
-    100, 311, 521, 721, 931, 1131, 1341, 1551, 1751, 1961, 2171, 2370, 2580, 2780, 2990, 3200,
+    85, 233, 360, 492, 649, 844, 965, 1094, 1380, 1562, 1710, 2040, 2197, 2557, 2766, 2814,
 ];
 
 pub trait CrosEcDriver {
@@ -225,6 +227,7 @@ impl Default for CrosEc {
 /// Find out which drivers are available
 ///
 /// Depending on the availability we choose the first one as default
+#[allow(clippy::vec_init_then_push)]
 fn available_drivers() -> Vec<CrosEcDriverType> {
     let mut drivers = vec![];
 
@@ -298,13 +301,16 @@ impl CrosEc {
         }
     }
 
-    pub fn cmd_version_supported(&self, cmd: u16, version: u8) -> EcResult<bool> {
-        let res = EcRequestGetCmdVersionsV1 { cmd: cmd.into() }.send_command(self);
+    pub fn cmd_version_supported(&self, cmd: u32, version: u8) -> EcResult<bool> {
+        let res = EcRequestGetCmdVersionsV1 { cmd }.send_command(self);
         let mask = if let Ok(res) = res {
             res.version_mask
-        } else {
-            let res = EcRequestGetCmdVersionsV0 { cmd: cmd as u8 }.send_command(self)?;
+        } else if let Ok(cmd) = u8::try_from(cmd) {
+            // V0 only supports 8-bit command IDs, so only fall back for commands <= 255
+            let res = EcRequestGetCmdVersionsV0 { cmd }.send_command(self)?;
             res.version_mask
+        } else {
+            return Ok(false);
         };
 
         Ok(mask & (1 << version) > 0)
@@ -547,6 +553,67 @@ impl CrosEc {
         })
     }
 
+    /// Check if the retimer is in firmware update mode
+    ///
+    /// This is normally false. Update mode is used to enable the retimer power even when no device
+    /// is attached.
+    pub fn retimer_in_fwupd_mode(&self, retimer: u8) -> EcResult<bool> {
+        let status = EcRequestRetimerControl {
+            controller: retimer,
+            mode: RetimerControlMode::CheckStatus as u8,
+        }
+        .send_command(self)?;
+
+        Ok(status.status == 0x01)
+    }
+
+    /// Enable or disable retimer update mode
+    ///
+    /// Check for success and returns Err if not successful
+    pub fn retimer_enable_fwupd(&self, retimer: u8, enable: bool) -> EcResult<()> {
+        if self.retimer_in_fwupd_mode(retimer)? == enable {
+            info!("Retimer update mode already: {:?}", enable);
+            return Ok(());
+        }
+        EcRequestRetimerControl {
+            controller: retimer,
+            mode: if enable {
+                RetimerControlMode::EntryFwUpdateMode as u8
+            } else {
+                RetimerControlMode::ExitFwUpdateMode as u8
+            },
+        }
+        .send_command(self)?;
+
+        // Wait half a second to let it enter the new mode
+        os_specific::sleep(500_000);
+
+        if self.retimer_in_fwupd_mode(retimer).unwrap() != enable {
+            error!("Failed to set retimer update mode to: {}", enable);
+            return Err(EcError::DeviceError(format!(
+                "Failed to set retimer update mode to: {}",
+                enable
+            )));
+        }
+        Ok(())
+    }
+
+    /// Enable or disable retimer Thunderbolt compliance mode (Intel only)
+    ///
+    /// Cannot check for success
+    pub fn retimer_enable_compliance(&self, retimer: u8, enable: bool) -> EcResult<()> {
+        EcRequestRetimerControl {
+            controller: retimer,
+            mode: if enable {
+                RetimerControlMode::EnableComplianceMode as u8
+            } else {
+                RetimerControlMode::DisableComplianceMode as u8
+            },
+        }
+        .send_command(self)?;
+        Ok(())
+    }
+
     pub fn get_input_deck_status(&self) -> EcResult<InputDeckStatus> {
         let status = EcRequestDeckState {
             mode: DeckStateMode::ReadOnly,
@@ -594,8 +661,30 @@ impl CrosEc {
 
         println!("Input Deck");
         println!("  Chassis Closed:      {}", !intrusion.currently_open);
-        println!("  Audio Daughterboard: {}", is_present(audio.is_some()));
-        println!("  Touchpad:            {}", is_present(tp.is_some()));
+        println!(
+            "  Audio Daughterboard: {}",
+            if let Some(audio) = audio {
+                format!("{} ({})", is_present(true), audio)
+            } else {
+                is_present(false).to_string()
+            }
+        );
+        println!(
+            "    ADC Value (mV)     {:?}",
+            self.adc_read(Framework13Adc::AudioBoardId as u8)
+        );
+        println!(
+            "  Touchpad:            {}",
+            if let Some(tp) = tp {
+                format!("{} ({})", is_present(true), tp)
+            } else {
+                is_present(false).to_string()
+            }
+        );
+        println!(
+            "    ADC Value (mV)     {:?}",
+            self.adc_read(Framework13Adc::TouchpadBoardId as u8)
+        );
 
         Ok(())
     }
@@ -707,9 +796,25 @@ impl CrosEc {
     /// | 79000 | 79FFF | 01000 | Flash Flags |
     pub fn reflash(&self, data: &[u8], ft: EcFlashType, dry_run: bool) -> EcResult<()> {
         let mut res = Ok(());
+
+        let cur_ver = self.version_info()?;
+        let platform = if let Some(ver) = ec_binary::parse_ec_version_str(&cur_ver) {
+            ver.platform
+        } else {
+            return Err(EcError::DeviceError(
+                "Cannot determine currently running platform.".to_string(),
+            ));
+        };
+
         if ft == EcFlashType::Full || ft == EcFlashType::Ro {
             if let Some(version) = ec_binary::read_ec_version(data, true) {
                 println!("EC RO Version in File: {:?}", version.version);
+                if version.details.platform != platform {
+                    return Err(EcError::DeviceError(format!(
+                        "RO Firmware file is for platform {}. Currently running {}",
+                        version.details.platform, platform
+                    )));
+                }
             } else {
                 return Err(EcError::DeviceError(
                     "File does not contain valid EC RO firmware".to_string(),
@@ -719,9 +824,15 @@ impl CrosEc {
         if ft == EcFlashType::Full || ft == EcFlashType::Rw {
             if let Some(version) = ec_binary::read_ec_version(data, false) {
                 println!("EC RW Version in File: {:?}", version.version);
+                if version.details.platform != platform {
+                    return Err(EcError::DeviceError(format!(
+                        "RW Firmware file is for platform {}. Currently running {}",
+                        version.details.platform, platform
+                    )));
+                }
             } else {
                 return Err(EcError::DeviceError(
-                    "File does not contain valid EW RO firmware".to_string(),
+                    "File does not contain valid EW RW firmware".to_string(),
                 ));
             }
         }
@@ -1188,7 +1299,8 @@ impl CrosEc {
             let remaining = len - data.len() as u16;
             let chunk_len = std::cmp::min(i2c_passthrough::MAX_I2C_CHUNK, remaining.into());
             let offset = addr + data.len() as u16;
-            let i2c_response = i2c_passthrough::i2c_read(
+            // Use 16-bit addressing for GPU EEPROM (required for larger EEPROMs)
+            let i2c_response = i2c_passthrough::i2c_read_16bit_addr(
                 self,
                 eeprom_port,
                 eeprom_addr,
@@ -1204,7 +1316,7 @@ impl CrosEc {
             data.extend(i2c_response.data);
         }
 
-        Ok(data)
+        Ok(data[..(len.into())].to_vec())
     }
 
     pub fn write_ec_gpu_chunk(&self, offset: u16, data: &[u8]) -> EcResult<()> {
@@ -1218,10 +1330,35 @@ impl CrosEc {
             "Writing GPU EEPROM {}",
             if dry_run { " (DRY RUN)" } else { "" }
         );
+        let mut force_power = false;
+
+        let info = EcRequestExpansionBayStatus {}.send_command(self)?;
+        println!("  Enabled:       {}", info.module_enabled());
+        println!("  No fault:      {}", !info.module_fault());
+        println!("  Door closed:   {}", info.hatch_switch_closed());
+
+        match info.expansion_bay_board() {
+            Ok(board) => println!("  Board:         {:?}", board),
+            Err(err) => println!("  Board:         {:?}", err),
+        }
+
+        if let Ok(ExpansionBayBoard::DualInterposer) = info.expansion_bay_board() {
+            /* Force power to the GPU if we are writing the EEPROM */
+            let res = self.set_gpio("gpu_3v_5v_en", true);
+            if let Err(err) = res {
+                error!("Failed to set ALW power to GPU off {:?}", err);
+                return Err(err);
+            }
+            println!("Forcing Power to GPU");
+            os_specific::sleep(100_000);
+            force_power = true;
+        }
+
         // Need to program the EEPROM 32 bytes at a time.
         let chunk_size = 32;
 
         let chunks = data.len() / chunk_size;
+        let mut return_val: EcResult<()> = Ok(());
         for chunk_no in 0..chunks {
             let offset = chunk_no * chunk_size;
             // Current chunk might be smaller if it's the last
@@ -1248,12 +1385,26 @@ impl CrosEc {
             // Don't read too fast, wait 100ms before writing more to allow for page erase/write cycle.
             os_specific::sleep(100_000);
             if let Err(err) = res {
-                println!("  Failed to write chunk: {:?}", err);
-                return Err(err);
+                error!("Failed to write chunk: {:?}", err);
+                return_val = Err(err);
+                break;
             }
         }
         println!();
-        Ok(())
+
+        if force_power {
+            let res = self.set_gpio("gpu_3v_5v_en", false);
+            if let Err(err) = res {
+                error!("Failed to set ALW power to GPU off {:?}", err);
+                return if return_val.is_err() {
+                    return_val
+                } else {
+                    Err(err)
+                };
+            }
+        };
+
+        return_val
     }
 
     pub fn read_gpu_descriptor(&self) -> EcResult<Vec<u8>> {
@@ -1391,6 +1542,19 @@ impl CrosEc {
         Ok(res.hibernation_delay)
     }
 
+    pub fn reset_s0ix_counter(&self) -> EcResult<()> {
+        EcRequestS0ixCounter {
+            flags: EC_S0IX_COUNTER_RESET,
+        }
+        .send_command(self)?;
+        Ok(())
+    }
+
+    pub fn get_s0ix_counter(&self) -> EcResult<u32> {
+        let res = EcRequestS0ixCounter { flags: 0 }.send_command(self)?;
+        Ok(res.s0ix_counter)
+    }
+
     /// Check features supported by the firmware
     pub fn get_features(&self) -> EcResult<()> {
         let data = EcRequestGetFeatures {}.send_command(self)?;
@@ -1459,6 +1623,19 @@ impl CrosEc {
         .send_command(self)
     }
 
+    pub fn set_gpio(&self, name: &str, value: bool) -> EcResult<()> {
+        const MAX_LEN: usize = 32;
+        let mut request = EcRequestGpioSetV0 {
+            name: [0; MAX_LEN],
+            value: value as u8,
+        };
+
+        let end = MAX_LEN.min(name.len());
+        request.name[..end].copy_from_slice(&name.as_bytes()[..end]);
+
+        request.send_command(self)?;
+        Ok(())
+    }
     pub fn get_gpio(&self, name: &str) -> EcResult<bool> {
         const MAX_LEN: usize = 32;
         let mut request = EcRequestGpioGetV0 { name: [0; MAX_LEN] };
@@ -1565,6 +1742,64 @@ impl CrosEc {
         }
         Ok(())
     }
+
+    pub fn read_board_id_hc(&self, board_id_type: BoardIdType) -> EcResult<Option<u8>> {
+        let res = EcRequestReadBoardId {
+            board_id_type: board_id_type as u8,
+        }
+        .send_command(self)?;
+        match res.board_id {
+            -1 => Err(EcError::DeviceError(format!(
+                "Failed to read Board ID {:?}",
+                board_id_type
+            ))),
+            15 => Ok(None),
+            0..=14 => Ok(Some(res.board_id as u8)),
+            n => Err(EcError::DeviceError(format!("Invalid Board ID {}", n))),
+        }
+    }
+
+    pub fn get_uptime_info(&self) -> EcResult<()> {
+        let res = EcRequestGetUptimeInfo {}.send_command(self)?;
+        let t_since_boot = Duration::from_millis(res.time_since_ec_boot.into());
+        println!("EC Uptime");
+        println!(
+            "  Time since EC Boot:      {}",
+            util::format_duration(&t_since_boot)
+        );
+        println!("  AP Resets since EC Boot: {}", {
+            res.ap_resets_since_ec_boot
+        });
+        println!("  EC Reset Flags");
+        for flag in 0..(EcResetFlag::Count as usize) {
+            if ((1 << flag) & res.ec_reset_flags) > 0 {
+                // Safe to unwrap unless coding mistake
+                println!(
+                    "    {:?}",
+                    <EcResetFlag as FromPrimitive>::from_usize(flag).unwrap()
+                );
+            }
+        }
+        println!("  Recent AP Resets");
+        for reset in res.recent_ap_resets {
+            if reset.reset_time_ms == 0 {
+                // Empty entry
+                continue;
+            }
+            let reset_time = Duration::from_millis(reset.reset_time_ms.into());
+            println!(
+                "      Reset Time:          {}",
+                util::format_duration(&reset_time)
+            );
+            let reset_cause: Option<ResetCause> = FromPrimitive::from_u16(reset.reset_cause);
+            if let Some(cause) = reset_cause {
+                println!("            Cause:         {:?}", cause);
+            } else {
+                println!("            Cause:         Unknown");
+            }
+        }
+        Ok(())
+    }
 }
 
 #[cfg_attr(not(feature = "uefi"), derive(clap::ValueEnum))]
