@@ -7,7 +7,7 @@ use alloc::format;
 use alloc::string::String;
 use alloc::string::ToString;
 use alloc::vec::Vec;
-use guid_create::{Guid, GUID};
+use guid_create::{CGuid, GUID};
 use log::Level;
 use num_traits::FromPrimitive;
 
@@ -35,6 +35,7 @@ use crate::ccgx::device::{FwMode, PdController, PdPort};
 use crate::ccgx::hid::{check_ccg_fw_version, find_devices, DP_CARD_PID, HDMI_CARD_PID};
 use crate::ccgx::{self, MainPdVersions, PdVersions, SiliconId::*};
 use crate::chromium_ec;
+use crate::chromium_ec::commands::BoardIdType;
 use crate::chromium_ec::commands::DeckStateMode;
 use crate::chromium_ec::commands::FpLedBrightnessLevel;
 use crate::chromium_ec::commands::RebootEcCmd;
@@ -49,6 +50,10 @@ use crate::ec_binary;
 use crate::esrt;
 #[cfg(feature = "rusb")]
 use crate::inputmodule::check_inputmodule_version;
+#[cfg(target_os = "linux")]
+use crate::nvme;
+use crate::os_specific;
+use crate::parade_retimer;
 use crate::power;
 use crate::smbios;
 use crate::smbios::ConfigDigit0;
@@ -59,6 +64,8 @@ use crate::touchpad::print_touchpad_fw_ver;
 use crate::touchscreen;
 #[cfg(feature = "uefi")]
 use crate::uefi::enable_page_break;
+#[cfg(feature = "rusb")]
+use crate::usbhub::check_usbhub_version;
 use crate::util::{self, Config, Platform, PlatformFamily};
 #[cfg(feature = "hidapi")]
 use hidapi::HidApi;
@@ -66,6 +73,9 @@ use sha2::{Digest, Sha256, Sha384, Sha512};
 //use smbioslib::*;
 use smbioslib::{DefinedStruct, SMBiosInformation};
 
+#[cfg(feature = "nvidia")]
+use nvml_wrapper::{enum_wrappers::device::TemperatureSensor, Nvml};
+
 use crate::chromium_ec::{CrosEc, CrosEcDriverType, HardwareDeviceType};
 
 #[cfg(feature = "uefi")]
@@ -161,7 +171,7 @@ pub struct Cli {
     pub sensors: bool,
     pub fansetduty: Option<(Option<u32>, u32)>,
     pub fansetrpm: Option<(Option<u32>, u32)>,
-    pub autofanctrl: bool,
+    pub autofanctrl: Option<Option<u8>>,
     pub pdports: bool,
     pub privacy: bool,
     pub pd_info: bool,
@@ -182,6 +192,8 @@ pub struct Cli {
     pub flash_rw_ec: Option<String>,
     pub driver: Option<CrosEcDriverType>,
     pub test: bool,
+    pub test_retimer: bool,
+    pub boardid: bool,
     pub dry_run: bool,
     pub force: bool,
     pub intrusion: bool,
@@ -204,6 +216,8 @@ pub struct Cli {
     pub console: Option<ConsoleArg>,
     pub reboot_ec: Option<RebootEcArg>,
     pub ec_hib_delay: Option<Option<u32>>,
+    pub uptimeinfo: bool,
+    pub s0ix_counter: bool,
     pub hash: Option<String>,
     pub pd_addrs: Option<(u16, u16, u16)>,
     pub pd_ports: Option<(u8, u8, u8)>,
@@ -212,6 +226,7 @@ pub struct Cli {
     pub flash_gpu_descriptor: Option<(u8, String)>,
     pub flash_gpu_descriptor_file: Option<String>,
     pub dump_gpu_descriptor_file: Option<String>,
+    pub nvidia: bool,
     // UEFI only
     pub allupdate: bool,
     pub paginate: bool,
@@ -265,6 +280,8 @@ pub fn parse(args: &[String]) -> Cli {
             // flash_rw_ec
             driver: cli.driver,
             test: cli.test,
+            test_retimer: cli.test_retimer,
+            boardid: cli.boardid,
             dry_run: cli.dry_run,
             // force
             intrusion: cli.intrusion,
@@ -287,6 +304,8 @@ pub fn parse(args: &[String]) -> Cli {
             console: cli.console,
             reboot_ec: cli.reboot_ec,
             // ec_hib_delay
+            uptimeinfo: cli.uptimeinfo,
+            s0ix_counter: cli.s0ix_counter,
             hash: cli.hash,
             pd_addrs: cli.pd_addrs,
             pd_ports: cli.pd_ports,
@@ -294,6 +313,7 @@ pub fn parse(args: &[String]) -> Cli {
             info: cli.info,
             // flash_gpu_descriptor
             // flash_gpu_descriptor_file
+            nvidia: cli.nvidia,
             // allupdate
             paginate: cli.paginate,
             // raw_command
@@ -461,6 +481,8 @@ fn print_stylus_battery_level() {
 }
 
 fn print_versions(ec: &CrosEc) {
+    println!("Tool Version:     {}", built_info::PKG_VERSION);
+    println!("OS Version:       {}", os_specific::get_os_version());
     println!("Mainboard Hardware");
     if let Some(ver) = smbios::get_product_name() {
         println!("  Type:           {}", ver);
@@ -475,9 +497,12 @@ fn print_versions(ec: &CrosEc) {
     println!("UEFI BIOS");
     if let Some(smbios) = get_smbios() {
         let bios_entries = smbios.collect::<SMBiosInformation>();
-        let bios = bios_entries.first().unwrap();
-        println!("  Version:        {}", bios.version());
-        println!("  Release Date:   {}", bios.release_date());
+        if let Some(bios) = bios_entries.first() {
+            println!("  Version:        {}", bios.version());
+            println!("  Release Date:   {}", bios.release_date());
+        } else {
+            println!("  Version:        Unknown");
+        }
     } else {
         println!("  Version:        Unknown");
     }
@@ -672,7 +697,7 @@ fn print_versions(ec: &CrosEc) {
         }
     }
     if has_retimer {
-        println!("Retimers");
+        println!("Intel Retimers");
         if let Some(fw_version) = left_retimer {
             println!("  Left:           0x{:X} ({})", fw_version, fw_version);
         }
@@ -684,6 +709,27 @@ fn print_versions(ec: &CrosEc) {
             println!("  Unknown");
         }
     }
+    match parade_retimer::get_version(ec) {
+        // Does not exist
+        Ok(None) => {}
+        Ok(Some(ver)) => {
+            println!("Parade Retimers");
+            if let [a, b, c, d, ..] = ver.as_slice() {
+                println!("  dGPU:           {:X}.{:X}.{:X}.{:X}", a, b, c, d);
+            } else {
+                println!("  dGPU:           Unknown");
+            }
+        }
+        _err => {
+            // Only Framework 16 has dGPU support (which has Parade Retimer)
+            if smbios::get_platform().and_then(Platform::which_family)
+                == Some(PlatformFamily::Framework16)
+            {
+                println!("Parade Retimers");
+                println!("  Unknown");
+            }
+        }
+    }
 
     #[cfg(target_os = "linux")]
     if smbios::get_platform().and_then(Platform::which_cpu_vendor) != Some(util::CpuVendor::Amd) {
@@ -702,6 +748,9 @@ fn print_versions(ec: &CrosEc) {
     #[cfg(feature = "rusb")]
     let _ignore_err = check_camera_version();
 
+    #[cfg(feature = "rusb")]
+    let _ignore_err = check_usbhub_version();
+
     #[cfg(feature = "rusb")]
     let _ignore_err = check_inputmodule_version();
 
@@ -714,6 +763,253 @@ fn print_versions(ec: &CrosEc) {
     }
     #[cfg(feature = "hidapi")]
     print_dp_hdmi_details(false);
+
+    #[cfg(target_os = "linux")]
+    for i in 0..4 {
+        let device = format!("/dev/nvme{i}");
+        match nvme::get_nvme_firmware_version(&device) {
+            Ok(dev) => {
+                println!("NVMe Device: {}", device);
+                println!(" Model Number:     {}", dev.model_number);
+                println!(" Firmware Version: {}", dev.firmware_version);
+            }
+            Err(_e) => {
+                // TODO: Maybe print errors but ignore "Not such file or directory"
+                // eprintln!("Failed to get firmware version for {}: {}", device, e);
+            }
+        }
+    }
+
+    #[cfg(feature = "nvidia")]
+    print_nvidia_details();
+}
+
+#[cfg(feature = "nvidia")]
+fn probably_has_nvidia() -> bool {
+    match smbios::get_platform().and_then(Platform::which_family) {
+        Some(PlatformFamily::Framework12) => false,
+        Some(PlatformFamily::Framework13) => false,
+        Some(PlatformFamily::FrameworkDesktop) => true,
+        Some(PlatformFamily::Framework16) => true,
+        _ => true,
+    }
+}
+
+/// Brief NVIDIA details for --version output
+#[cfg(feature = "nvidia")]
+fn print_nvidia_details() {
+    let probably_has_nvidia = probably_has_nvidia();
+    let nvml = match Nvml::init() {
+        Ok(nvml) => nvml,
+        Err(err) => {
+            if probably_has_nvidia {
+                error!("Nvidia, library init fail: {:?}", err);
+            }
+            return;
+        }
+    };
+    let device = match nvml.device_by_index(0) {
+        Ok(device) => device,
+        Err(err) => {
+            if probably_has_nvidia {
+                error!("Nvidia, device not found: {:?}", err);
+            }
+            return;
+        }
+    };
+
+    println!("NVIDIA GPU");
+    println!(
+        "  Name:           {}",
+        device.name().unwrap_or("Unknown".to_string())
+    );
+    println!(
+        "  VBIOS Version:  {}",
+        device.vbios_version().unwrap_or("Unknown".to_string())
+    );
+}
+
+/// Detailed NVIDIA information for --nvidia command
+#[cfg(feature = "nvidia")]
+fn print_nvidia_info() {
+    let probably_has_nvidia = probably_has_nvidia();
+    let nvml = match Nvml::init() {
+        Ok(nvml) => nvml,
+        Err(err) => {
+            if probably_has_nvidia {
+                error!("Nvidia, library init fail: {:?}", err);
+            }
+            return;
+        }
+    };
+    let device = match nvml.device_by_index(0) {
+        Ok(device) => device,
+        Err(err) => {
+            if probably_has_nvidia {
+                error!("Nvidia, device not found: {:?}", err);
+            }
+            return;
+        }
+    };
+
+    println!("NVIDIA GPU");
+
+    // Basic identification
+    println!("Identification");
+    println!(
+        "  Name:              {}",
+        device.name().unwrap_or("Unknown".to_string())
+    );
+    if let Ok(arch) = device.architecture() {
+        println!("  Architecture:      {:?}", arch);
+    }
+    if let Ok(serial) = device.serial() {
+        println!("  Serial Number:     {}", serial);
+    }
+    if let Ok(part_number) = device.board_part_number() {
+        println!("  Part Number:       {}", part_number);
+    }
+    if let Ok(board_id) = device.board_id() {
+        println!("  Board ID:          {}", board_id);
+    }
+
+    // Firmware versions
+    println!("Firmware");
+    println!(
+        "  VBIOS Version:     {}",
+        device.vbios_version().unwrap_or("Unknown".to_string())
+    );
+    println!(
+        "  InfoROM Version:   {}",
+        device
+            .info_rom_image_version()
+            .unwrap_or("Unknown".to_string())
+    );
+
+    // PCI information
+    println!("PCI");
+    if let Ok(pci) = device.pci_info() {
+        println!("  Bus:               {:02X}", pci.bus);
+        println!("  Device:            {:02X}", pci.device);
+        println!("  Domain:            {:04X}", pci.domain);
+        println!("  Device ID:         {:04X}", pci.pci_device_id);
+        if let Some(sub_id) = pci.pci_sub_system_id {
+            println!("  Subsystem ID:      {:08X}", sub_id);
+        }
+    }
+
+    // Power information
+    println!("Power");
+    if let Ok(state) = device.performance_state() {
+        println!("  Performance State: {:?}", state);
+    }
+    if let Ok(power) = device.power_usage() {
+        println!("  Current Usage:     {:.2} W", power as f64 / 1000.0);
+    }
+    if let Ok(limit) = device.power_management_limit() {
+        println!("  Power Limit:       {:.2} W", limit as f64 / 1000.0);
+    }
+    if let Ok(default) = device.power_management_limit_default() {
+        println!("  Default Limit:     {:.2} W", default as f64 / 1000.0);
+    }
+    if let Ok(constraints) = device.power_management_limit_constraints() {
+        println!(
+            "  Min Limit:         {:.2} W",
+            constraints.min_limit as f64 / 1000.0
+        );
+        println!(
+            "  Max Limit:         {:.2} W",
+            constraints.max_limit as f64 / 1000.0
+        );
+    }
+    if let Ok(energy) = device.total_energy_consumption() {
+        println!("  Total Energy:      {:.2} J", energy as f64 / 1000.0);
+    }
+
+    // Thermal information
+    println!("Thermal");
+    if let Ok(temp) = device.temperature(TemperatureSensor::Gpu) {
+        println!("  GPU Temperature:   {}C", temp);
+    }
+    if let Ok(num_fans) = device.num_fans() {
+        println!("  Number of Fans:    {}", num_fans);
+    }
+
+    // Throttling
+    if let Ok(throttle) = device.current_throttle_reasons() {
+        println!("Throttle Reasons");
+        if throttle.is_empty() {
+            println!("  None");
+        } else {
+            if throttle.contains(nvml_wrapper::bitmasks::device::ThrottleReasons::GPU_IDLE) {
+                println!("  GPU Idle");
+            }
+            if throttle.contains(
+                nvml_wrapper::bitmasks::device::ThrottleReasons::APPLICATIONS_CLOCKS_SETTING,
+            ) {
+                println!("  Applications Clocks Setting");
+            }
+            if throttle.contains(nvml_wrapper::bitmasks::device::ThrottleReasons::SW_POWER_CAP) {
+                println!("  Software Power Cap");
+            }
+            if throttle.contains(nvml_wrapper::bitmasks::device::ThrottleReasons::HW_SLOWDOWN) {
+                println!("  Hardware Slowdown");
+            }
+            if throttle.contains(nvml_wrapper::bitmasks::device::ThrottleReasons::SYNC_BOOST) {
+                println!("  Sync Boost");
+            }
+            if throttle
+                .contains(nvml_wrapper::bitmasks::device::ThrottleReasons::SW_THERMAL_SLOWDOWN)
+            {
+                println!("  Software Thermal Slowdown");
+            }
+            if throttle
+                .contains(nvml_wrapper::bitmasks::device::ThrottleReasons::HW_THERMAL_SLOWDOWN)
+            {
+                println!("  Hardware Thermal Slowdown");
+            }
+            if throttle
+                .contains(nvml_wrapper::bitmasks::device::ThrottleReasons::HW_POWER_BRAKE_SLOWDOWN)
+            {
+                println!("  Hardware Power Brake Slowdown");
+            }
+            if throttle
+                .contains(nvml_wrapper::bitmasks::device::ThrottleReasons::DISPLAY_CLOCK_SETTING)
+            {
+                println!("  Display Clock Setting");
+            }
+        }
+    }
+
+    // Utilization
+    println!("Utilization");
+    if let Ok(util) = device.utilization_rates() {
+        println!("  GPU:               {}%", util.gpu);
+        println!("  Memory:            {}%", util.memory);
+    }
+
+    // Memory information
+    println!("Memory");
+    if let Ok(mem) = device.memory_info() {
+        let total_gb = mem.total as f64 / (1024.0 * 1024.0 * 1024.0);
+        let used_gb = mem.used as f64 / (1024.0 * 1024.0 * 1024.0);
+        let free_gb = mem.free as f64 / (1024.0 * 1024.0 * 1024.0);
+        println!("  Total:             {:.2} GB", total_gb);
+        println!("  Used:              {:.2} GB", used_gb);
+        println!("  Free:              {:.2} GB", free_gb);
+    }
+
+    // Display status
+    println!("Display");
+    if let Ok(active) = device.is_display_active() {
+        println!("  Active:            {}", if active { "Yes" } else { "No" });
+    }
+    if let Ok(connected) = device.is_display_connected() {
+        println!(
+            "  Connected:         {}",
+            if connected { "Yes" } else { "No" }
+        );
+    }
 }
 
 fn print_esrt() {
@@ -769,18 +1065,66 @@ fn dump_ec_flash(ec: &CrosEc, dump_path: &str) {
 }
 
 fn dump_dgpu_eeprom(ec: &CrosEc, dump_path: &str) {
-    let flash_bin = ec.read_gpu_descriptor().unwrap();
+    // Read raw bytes from EEPROM
+    let raw_bytes = match ec.read_ec_gpu_chunk(0x00, 256) {
+        Ok(data) => data,
+        Err(err) => {
+            error!("Failed to read EEPROM: {:?}", err);
+            return;
+        }
+    };
+
+    // For stdout, just print the raw bytes
+    if dump_path == "-" {
+        println!("{:02X?}", raw_bytes);
+        println!("Read {} bytes from EEPROM", raw_bytes.len());
+        return;
+    }
+
+    // Check if header is valid by examining magic bytes
+    let expected_magic = [0x32, 0xAC, 0x00, 0x00];
+    let has_valid_header = raw_bytes.len() >= 4 && raw_bytes[0..4] == expected_magic;
+
+    let flash_bin = if has_valid_header {
+        // Header looks valid, try to read the full descriptor
+        match ec.read_gpu_descriptor() {
+            Ok(data) => data,
+            Err(err) => {
+                error!("GPU descriptor read failed: {:?}", err);
+                println!("Falling back to raw EEPROM dump (256 bytes)");
+                raw_bytes
+            }
+        }
+    } else {
+        error!(
+            "GPU descriptor invalid: magic {:02X?} != expected {:02X?}",
+            &raw_bytes[0..4],
+            expected_magic
+        );
+        println!("Dumping raw EEPROM (256 bytes)");
+        raw_bytes
+    };
 
     #[cfg(not(feature = "uefi"))]
     {
-        let mut file = fs::File::create(dump_path).unwrap();
-        file.write_all(&flash_bin).unwrap();
+        match fs::File::create(dump_path) {
+            Ok(mut file) => {
+                if let Err(err) = file.write_all(&flash_bin) {
+                    error!("Failed to write file: {:?}", err);
+                    return;
+                }
+            }
+            Err(err) => {
+                error!("Failed to create file: {:?}", err);
+                return;
+            }
+        }
     }
     #[cfg(feature = "uefi")]
     {
-        let ret = crate::uefi::fs::shell_write_file(dump_path, &flash_bin);
-        if ret.is_err() {
-            println!("Failed to dump EC FW image.");
+        if let Err(err) = crate::uefi::fs::shell_write_file(dump_path, &flash_bin) {
+            error!("Failed to dump EC FW image: {:?}", err);
+            return;
         }
     }
     println!("Wrote {} bytes to {}", flash_bin.len(), dump_path);
@@ -980,7 +1324,16 @@ pub fn run_with_args(args: &Cli, _allupdate: bool) -> i32 {
             Some(PlatformFamily::Framework12) => ec.print_fw12_inputdeck_status(),
             Some(PlatformFamily::Framework13) => ec.print_fw13_inputdeck_status(),
             Some(PlatformFamily::Framework16) => ec.print_fw16_inputdeck_status(),
-            _ => Ok(()),
+            // If we don't know which platform it is, we can use some heuristics
+            _ => {
+                // Only Framework 16 has this GPIO
+                if ec.get_gpio("sleep_l").is_ok() {
+                    ec.print_fw16_inputdeck_status()
+                } else {
+                    println!("  Unable to tell");
+                    Ok(())
+                }
+            }
         };
         print_err(res);
     } else if let Some(mode) = &args.inputdeck_mode {
@@ -1010,6 +1363,11 @@ pub fn run_with_args(args: &Cli, _allupdate: bool) -> i32 {
                 println!("    Hdr CRC:     {:X}", { header.crc32 });
             }
         }
+    } else if args.nvidia {
+        #[cfg(feature = "nvidia")]
+        print_nvidia_info();
+        #[cfg(not(feature = "nvidia"))]
+        error!("Not built with nvidia feature");
     } else if let Some(maybe_limit) = args.charge_limit {
         print_err(handle_charge_limit(&ec, maybe_limit));
     } else if let Some((limit, soc)) = args.charge_current_limit {
@@ -1116,6 +1474,14 @@ pub fn run_with_args(args: &Cli, _allupdate: bool) -> i32 {
             print_err(ec.set_ec_hib_delay(*delay));
         }
         print_err(ec.get_ec_hib_delay());
+    } else if args.uptimeinfo {
+        print_err(ec.get_uptime_info());
+    } else if args.s0ix_counter {
+        if let Some(counter) = print_err(ec.get_s0ix_counter()) {
+            println!("s0ix_counter: {}", counter);
+        } else {
+            println!("s0ix_counter: Unknown");
+        }
     } else if args.test {
         println!("Self-Test");
         let result = selftest(&ec);
@@ -1123,6 +1489,13 @@ pub fn run_with_args(args: &Cli, _allupdate: bool) -> i32 {
             println!("FAILED!!");
             return 1;
         }
+    } else if args.test_retimer {
+        println!("Retimer Self-Test");
+        if let Err(err) = selftest_retimer(&ec) {
+            println!("  Failed: {:?}", err);
+        }
+    } else if args.boardid {
+        print_board_ids(&ec);
     } else if args.power {
         return power::get_and_print_power_info(&ec);
     } else if args.thermal {
@@ -1133,7 +1506,9 @@ pub fn run_with_args(args: &Cli, _allupdate: bool) -> i32 {
         print_err(ec.fan_set_duty(fan, percent));
     } else if let Some((fan, rpm)) = args.fansetrpm {
         print_err(ec.fan_set_rpm(fan, rpm));
-    } else if args.autofanctrl {
+    } else if let Some(Some(fan_id)) = args.autofanctrl {
+        print_err(ec.autofanctrl(Some(fan_id)));
+    } else if let Some(None) = args.autofanctrl {
         print_err(ec.autofanctrl(None));
     } else if args.pdports {
         power::get_and_print_pd_info(&ec);
@@ -1258,7 +1633,7 @@ pub fn run_with_args(args: &Cli, _allupdate: bool) -> i32 {
             println!("  Size:       {:>20} B", data.len());
             println!("  Size:       {:>20} KB", data.len() / 1024);
             if let Some(header) = analyze_capsule(&data) {
-                if header.capsule_guid == Guid::from(esrt::WINUX_GUID) {
+                if header.capsule_guid == CGuid::from(esrt::WINUX_GUID) {
                     let ux_header = capsule::parse_ux_header(&data);
                     if let Some(dump_path) = &args.dump {
                         // TODO: Better error handling, rather than just panicking
@@ -1377,7 +1752,9 @@ pub fn run_with_args(args: &Cli, _allupdate: bool) -> i32 {
             println!("Unsupported on this platform");
         }
     } else if let Some(dump_path) = &args.dump_gpu_descriptor_file {
-        println!("Dumping to {}", dump_path);
+        if dump_path != "-" {
+            println!("Dumping to {}", dump_path);
+        }
         dump_dgpu_eeprom(&ec, dump_path);
     }
 
@@ -1406,7 +1783,7 @@ Options:
       --sensors              Print sensor information (ALS, G-Sensor)
       --fansetduty           Set fan duty cycle (0-100%)
       --fansetrpm            Set fan RPM (limited by EC fan table max RPM)
-      --autofanctrl          Turn on automatic fan speed control
+      --autofanctrl [<FANID>]Turn on automatic fan speed control (optionally provide fan index)
       --pdports              Show information about USB-C PD ports
       --info                 Show info from SMBIOS (Only on UEFI)
       --pd-info              Show details about the PD controllers
@@ -1425,6 +1802,7 @@ Options:
       --inputdeck            Show status of the input deck
       --inputdeck-mode       Set input deck power mode [possible values: auto, off, on] (Framework 16 only)
       --expansion-bay        Show status of the expansion bay (Framework 16 only)
+      --nvidia               Show NVIDIA GPU information (Framework 16 only)
       --charge-limit [<VAL>] Get or set battery charge limit (Percentage number as arg, e.g. '100')
       --charge-current-limit [<VAL>] Get or set battery current charge limit (Percentage number as arg, e.g. '100')
       --get-gpio <GET_GPIO>  Get GPIO value by name or all, if no name provided
@@ -1481,6 +1859,34 @@ fn hash(data: &[u8]) {
     util::print_buffer(sha512);
 }
 
+fn print_board_ids(ec: &CrosEc) {
+    println!("Board IDs");
+    println!(
+        "  Mainboard:    {:?}",
+        ec.read_board_id_hc(BoardIdType::Mainboard)
+    );
+    println!(
+        "  PowerButton:  {:?}",
+        ec.read_board_id_hc(BoardIdType::PowerButtonBoard)
+    );
+    println!(
+        "  Touchpad:     {:?}",
+        ec.read_board_id_hc(BoardIdType::Touchpad)
+    );
+    println!(
+        "  AudioBoard:   {:?}",
+        ec.read_board_id_hc(BoardIdType::AudioBoard)
+    );
+    println!(
+        "  dGPU0:        {:?}",
+        ec.read_board_id_hc(BoardIdType::DGpu0)
+    );
+    println!(
+        "  dGPU1:        {:?}",
+        ec.read_board_id_hc(BoardIdType::DGpu1)
+    );
+}
+
 fn selftest(ec: &CrosEc) -> Option<()> {
     if let Some(platform) = smbios::get_platform() {
         println!("  SMBIOS Platform:     {:?}", platform);
@@ -1553,6 +1959,67 @@ fn selftest(ec: &CrosEc) -> Option<()> {
     Some(())
 }
 
+// Platforms that have Retimers
+// Retimer I2C is always connected to the CPU, except for the Framework 16 dGPU retimer.
+//
+// - Framework 12
+//   - No Retimer, only retimer for both left ports (no firmware)
+// - Framework 13 Intel
+//   - One Intel retimer for each port (with firmware)
+// - Framework 13 AMD 7040
+//   - Kandou Retimer on top two ports (no firmware)
+//   - Analogix Retimer on bottom two ports (no firmware)
+// - Framework 13 AMD AI 300
+//   - Parade Retimer on top two ports (with firmware)
+//   - Analogix Retimer on bottom two ports (no firmware)
+// - Framework 16 AMD 7040
+//   - Kandou Retimer on top two ports (no firmware)
+//   - Analogix Retimer on lower and middle left ports (no firmware)
+// - Framework 16 AMD AI 300
+//   - Parade Retimer on top two ports (with firmware)
+// - Framework 16 AMD dGPU
+//   - None
+// - Framework 16 NVIDIA dGPU
+//   - Parade Retimer
+// - Framework Desktop
+//   - Parade Retimer on both back ports (with firmware)
+fn selftest_retimer(ec: &CrosEc) -> EcResult<()> {
+    // TODO: Make sure that it can work for the NVIDIA dGPU retimer and increase to 3
+    for i in 0..2 {
+        let update_mode = ec.retimer_in_fwupd_mode(i);
+        if update_mode == Err(EcError::Response(EcResponseStatus::InvalidParameter)) {
+            println!("  Retimer status not supported on this platform. Cannot test");
+            return Ok(());
+        }
+        println!("  Retimers on PD Controller {}", i);
+        println!(
+            "    In update mode: {:?}",
+            ec.retimer_in_fwupd_mode(i).unwrap()
+        );
+        if update_mode? {
+            println!("      Disabling it to start test.");
+            ec.retimer_enable_fwupd(i, false)?;
+        }
+
+        println!("    Enabling update mode");
+        let success = ec.retimer_enable_fwupd(i, true);
+        println!(
+            "    In update mode: {:?}",
+            ec.retimer_in_fwupd_mode(i).unwrap()
+        );
+
+        println!("    Disabling update mode");
+        ec.retimer_enable_fwupd(i, false)?;
+        os_specific::sleep(100);
+        println!(
+            "    In update mode: {:?}",
+            ec.retimer_in_fwupd_mode(i).unwrap()
+        );
+        success?;
+    }
+    Ok(())
+}
+
 fn smbios_info() {
     println!("Summary");
     println!("  Is Framework: {}", is_framework());
@@ -1671,7 +2138,6 @@ fn analyze_ccgx_pd_fw(data: &[u8]) {
 
         println!("FW 2");
         ccgx::binary::print_fw(&versions.main_fw);
-        return;
     } else if let Some(versions) = ccgx::binary::read_versions(data, Ccg6) {
         println!("Detected CCG6 firmware");
         println!("FW 1 (Backup)");
@@ -1679,7 +2145,6 @@ fn analyze_ccgx_pd_fw(data: &[u8]) {
 
         println!("FW 2 (Main)");
         ccgx::binary::print_fw(&versions.main_fw);
-        return;
     } else {
         println!("Failed to read PD versions")
     }
