@@ -39,7 +39,7 @@ pub fn parse(args: &[String]) -> Cli {
         sensors: false,
         fansetduty: None,
         fansetrpm: None,
-        autofanctrl: false,
+        autofanctrl: None,
         pdports: false,
         pd_info: false,
         pd_reset: None,
@@ -78,12 +78,16 @@ pub fn parse(args: &[String]) -> Cli {
         console: None,
         reboot_ec: None,
         ec_hib_delay: None,
+        uptimeinfo: false,
+        s0ix_counter: false,
         hash: None,
         // This is the only driver that works on UEFI
         driver: Some(CrosEcDriverType::Portio),
         pd_addrs: None,
         pd_ports: None,
         test: false,
+        test_retimer: false,
+        boardid: false,
         dry_run: false,
         force: false,
         help: false,
@@ -92,6 +96,7 @@ pub fn parse(args: &[String]) -> Cli {
         dump_gpu_descriptor_file: None,
         allupdate: false,
         info: false,
+        nvidia: false,
         raw_command: vec![],
     };
 
@@ -195,7 +200,19 @@ pub fn parse(args: &[String]) -> Cli {
             };
             found_an_option = true;
         } else if arg == "--autofanctrol" {
-            cli.autofanctrl = true;
+            cli.autofanctrl = if args.len() > i + 1 {
+                if let Ok(fan_id) = args[i + 1].parse::<u8>() {
+                    Some(Some(fan_id))
+                } else {
+                    println!(
+                        "Invalid value for --autofanctrl: '{}'. Must be integer < 256.",
+                        args[i + 1]
+                    );
+                    None
+                }
+            } else {
+                Some(None)
+            };
             found_an_option = true;
         } else if arg == "--pdports" {
             cli.pdports = true;
@@ -235,6 +252,9 @@ pub fn parse(args: &[String]) -> Cli {
         } else if arg == "--expansion-bay" {
             cli.expansion_bay = true;
             found_an_option = true;
+        } else if arg == "--nvidia" {
+            cli.nvidia = true;
+            found_an_option = true;
         } else if arg == "--charge-limit" {
             cli.charge_limit = if args.len() > i + 1 {
                 if let Ok(percent) = args[i + 1].parse::<u8>() {
@@ -463,7 +483,7 @@ pub fn parse(args: &[String]) -> Cli {
                 None
             };
             found_an_option = true;
-        } else if arg == "--reboot-ec" {
+        } else if arg == "--ec-hib-delay" {
             cli.ec_hib_delay = if args.len() > i + 1 {
                 if let Ok(delay) = args[i + 1].parse::<u32>() {
                     if delay == 0 {
@@ -480,9 +500,21 @@ pub fn parse(args: &[String]) -> Cli {
                 Some(None)
             };
             found_an_option = true;
+        } else if arg == "--uptimeinfo" {
+            cli.uptimeinfo = true;
+            found_an_option = true;
+        } else if arg == "--s0ix-counter" {
+            cli.s0ix_counter = true;
+            found_an_option = true;
         } else if arg == "-t" || arg == "--test" {
             cli.test = true;
             found_an_option = true;
+        } else if arg == "-t" || arg == "--test-retimer" {
+            cli.test_retimer = true;
+            found_an_option = true;
+        } else if arg == "--boardid" {
+            cli.boardid = true;
+            found_an_option = true;
         } else if arg == "-f" || arg == "--force" {
             cli.force = true;
             found_an_option = true;
