@@ -15,7 +15,7 @@ use log::{debug, error, info, trace};
 use std::prelude::v1::*;
 
 use core::prelude::v1::derive;
-use guid_create::{Guid, GUID};
+use guid_create::{CGuid, GUID};
 
 #[cfg(target_os = "linux")]
 use std::fs;
@@ -24,6 +24,11 @@ use std::io;
 #[cfg(target_os = "linux")]
 use std::path::Path;
 
+#[cfg(target_os = "windows")]
+use winreg::enums::HKEY_LOCAL_MACHINE;
+#[cfg(target_os = "windows")]
+use winreg::RegKey;
+
 #[cfg(target_os = "freebsd")]
 use nix::ioctl_readwrite;
 #[cfg(target_os = "freebsd")]
@@ -119,6 +124,12 @@ pub const FL16_BIOS_GUID: GUID = GUID::build_from_components(
     0x5d64,
     &[0x8e, 0x18, 0x65, 0x8d, 0x20, 0x5a, 0xcf, 0x34],
 );
+pub const AMD16_AI300_BIOS_GUID: GUID = GUID::build_from_components(
+    0x820436ee,
+    0x8208,
+    0x463b,
+    &[0x92, 0xb8, 0x82, 0x77, 0xd6, 0x38, 0x4d, 0x93],
+);
 pub const AMD13_RYZEN7040_BIOS_GUID: GUID = GUID::build_from_components(
     0xb5f7dcc1,
     0x568c,
@@ -186,6 +197,7 @@ pub enum FrameworkGuidKind {
     RplUCsme,
     MtlCsme,
     Fl16Bios,
+    Amd16Ai300Bios,
     Amd13Ryzen7040Bios,
     Amd13Ai300Bios,
     DesktopAmdAi300Bios,
@@ -193,7 +205,7 @@ pub enum FrameworkGuidKind {
     Unknown,
 }
 
-pub fn match_guid_kind(guid: &Guid) -> FrameworkGuidKind {
+pub fn match_guid_kind(guid: &CGuid) -> FrameworkGuidKind {
     match GUID::from(*guid) {
         TGL_BIOS_GUID => FrameworkGuidKind::TglBios,
         ADL_BIOS_GUID => FrameworkGuidKind::AdlBios,
@@ -201,6 +213,7 @@ pub fn match_guid_kind(guid: &Guid) -> FrameworkGuidKind {
         MTL_BIOS_GUID => FrameworkGuidKind::MtlBios,
         FW12_RPL_BIOS_GUID => FrameworkGuidKind::Fw12RplBios,
         FL16_BIOS_GUID => FrameworkGuidKind::Fl16Bios,
+        AMD16_AI300_BIOS_GUID => FrameworkGuidKind::Amd16Ai300Bios,
         AMD13_RYZEN7040_BIOS_GUID => FrameworkGuidKind::Amd13Ryzen7040Bios,
         AMD13_AI300_BIOS_GUID => FrameworkGuidKind::Amd13Ai300Bios,
         DESKTOP_AMD_AI300_BIOS_GUID => FrameworkGuidKind::DesktopAmdAi300Bios,
@@ -292,7 +305,7 @@ impl UpdateStatus {
 // TODO: Decode into proper Rust types
 #[derive(Clone)]
 pub struct EsrtResourceEntry {
-    pub fw_class: Guid,
+    pub fw_class: CGuid,
     pub fw_type: u32, // ResourceType
     pub fw_version: u32,
     pub lowest_supported_fw_version: u32,
@@ -364,7 +377,7 @@ fn esrt_from_sysfs(dir: &Path) -> io::Result<Esrt> {
                 let last_attempt_version = fs::read_to_string(path.join("last_attempt_version"))?;
                 let last_attempt_status = fs::read_to_string(path.join("last_attempt_status"))?;
                 let esrt = EsrtResourceEntry {
-                    fw_class: Guid::from(
+                    fw_class: CGuid::from(
                         GUID::parse(fw_class.trim()).expect("Kernel provided wrong value"),
                     ),
                     fw_type: fw_type
@@ -417,67 +430,28 @@ pub fn get_esrt() -> Option<Esrt> {
         resource_version: ESRT_FIRMWARE_RESOURCE_VERSION,
         entries: vec![],
     };
-    use wmi::*;
-    debug!("Opening WMI");
-    let wmi_con = WMIConnection::new(COMLibrary::new().unwrap()).unwrap();
-    use std::collections::HashMap;
-    use wmi::Variant;
-    debug!("Querying WMI");
-    let results: Vec<HashMap<String, Variant>> = wmi_con.raw_query("SELECT HardwareID, Name FROM Win32_PnPEntity WHERE ClassGUID = '{f2e7dd72-6468-4e36-b6f1-6488f42c1b52}'").unwrap();
 
-    let re = regex::Regex::new(r"([\-a-h0-9]+)\}&REV_([A-F0-9]+)").expect("Bad regex");
-    for (i, val) in results.iter().enumerate() {
-        let hwid = &val["HardwareID"];
-        if let Variant::Array(strs) = hwid {
-            if let Variant::String(s) = &strs[0] {
-                // Sample "UEFI\\RES_{c57fd615-2ac9-4154-bf34-4dc715344408}&REV_CF"
-                let caps = re.captures(s).expect("No caps");
-                let guid_str = caps.get(1).unwrap().as_str().to_string();
-                let ver_str = caps.get(2).unwrap().as_str().to_string();
+    let hklm = RegKey::predef(HKEY_LOCAL_MACHINE);
+    let esrt = hklm.open_subkey("HARDWARE\\UEFI\\ESRT").ok()?;
+    for subkey in esrt.enum_keys() {
+        let guid = subkey.ok()?;
+        let short_guid = guid.trim_matches(|c| c == '{' || c == '}');
+        let esrt_entry = esrt.open_subkey(&guid).ok()?;
+        let esrt = EsrtResourceEntry {
+            fw_class: GUID::parse(short_guid).ok()?.into(),
+            fw_type: esrt_entry.get_value("Type").ok()?,
+            fw_version: esrt_entry.get_value("Version").ok()?,
+            lowest_supported_fw_version: esrt_entry.get_value("LowestSupportedVersion").ok()?,
+            capsule_flags: 0,
+            last_attempt_version: esrt_entry.get_value("LastAttemptVersion").ok()?,
+            last_attempt_status: esrt_entry.get_value("LastAttemptStatus").ok()?,
+        };
 
-                let guid = GUID::parse(guid_str.trim()).expect("Kernel provided wrong value");
-                let guid_kind = match_guid_kind(&Guid::from(guid));
-                let ver = u32::from_str_radix(&ver_str, 16).unwrap();
-                debug!("ESRT Entry {}", i);
-                debug!("  Name:    {:?}", guid_kind);
-                debug!("  GUID:    {}", guid_str);
-                debug!("  Version: {:X} ({})", ver, ver);
-
-                let fw_type = if let Variant::String(name) = &val["Name"] {
-                    match name.as_str() {
-                        "System Firmware" => 1,
-                        "Device Firmware" => 2,
-                        _ => 0,
-                    }
-                } else {
-                    0
-                };
-
-                // TODO: The missing fields are present in Device Manager
-                // So there must be a way to get at them
-                let esrt = EsrtResourceEntry {
-                    fw_class: Guid::from(guid),
-                    fw_type,
-                    fw_version: ver,
-                    // TODO: Not exposed by windows
-                    lowest_supported_fw_version: 0,
-                    // TODO: Not exposed by windows
-                    capsule_flags: 0,
-                    // TODO: Not exposed by windows
-                    last_attempt_version: 0,
-                    // TODO: Not exposed by windows
-                    last_attempt_status: 0,
-                };
-                esrt_table.resource_count += 1;
-                esrt_table.resource_count_max += 1;
-                esrt_table.entries.push(esrt);
-            } else {
-                error!("Strs: {:#?}", strs[0]);
-            }
-        } else {
-            error!("{:#?}", hwid);
-        }
+        esrt_table.resource_count += 1;
+        esrt_table.resource_count_max += 1;
+        esrt_table.entries.push(esrt);
     }
+
     Some(esrt_table)
 }
 
@@ -543,7 +517,7 @@ pub fn get_esrt() -> Option<Esrt> {
     for table in config_tables {
         // TODO: Why aren't they the same type?
         //debug!("Table: {:?}", table);
-        let table_guid: Guid = unsafe { std::mem::transmute(table.guid) };
+        let table_guid: CGuid = unsafe { std::mem::transmute(table.guid) };
         let table_guid = GUID::from(table_guid);
         match table_guid {
             SYSTEM_RESOURCE_TABLE_GUID => unsafe {
