@@ -19,12 +19,16 @@ pub mod audio_card;
 pub mod camera;
 #[cfg(feature = "rusb")]
 pub mod inputmodule;
+#[cfg(target_os = "linux")]
+pub mod nvme;
 #[cfg(feature = "hidapi")]
 pub mod touchpad;
 #[cfg(feature = "hidapi")]
 pub mod touchscreen;
 #[cfg(all(feature = "hidapi", windows))]
 pub mod touchscreen_win;
+#[cfg(feature = "rusb")]
+pub mod usbhub;
 
 #[cfg(feature = "uefi")]
 #[macro_use]
@@ -39,6 +43,7 @@ pub mod csme;
 pub mod ec_binary;
 pub mod esrt;
 mod os_specific;
+pub mod parade_retimer;
 pub mod power;
 pub mod smbios;
 #[cfg(feature = "uefi")]
